//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Class to display a density function (cdf or pdf)
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
class DensityFuncDisplay{
    float _wide = 300;
    float _high = 50;
    float _scale = 25;
    float _pad = 5;
    String _placement;
    boolean _cdfOn = false;

    // * * * Constructor * * * //
    DensityFuncDisplay(String placement) { _placement = placement; }

    // * * * Constructor (default placement) * * * //
    DensityFuncDisplay() { this("RIGHT"); }

    // * * * Toggle the display * * * //
    void toggleDisplay() { _cdfOn = !_cdfOn; }

    // * * * Draw the pdf * * * //
    void drawDensityFunc(DataObject object, int id, DataObject comp){
	
	if(object.getNumPts() != -1){
	    double [] data = object.getHistogram(id);
	    double maxDen  = comp.getMaxDensity(id);
            double tmp     = object.getMaxDensity(id);
            maxDen  = Math.max(tmp, maxDen); 
	    double minDen  = object.getMinDensity(id);
	    double imean   = object.getMean(id);
	    double minDat  = object.getMinData(id);
	    double maxDat  = object.getMaxData(id);
	    // Get the dimensions
	    int numBins =  data.length;
	
	    // Translate to the local origin
	    pushMatrix();	
	    translate(0, height-_high-(_pad*5), 0);
	    if(_placement == "LEFT")
		translate(width-_wide-4*_pad, 0, 0);
	    else
		translate(4*_pad, 0, 0);
	  
	    // Draw the rectangle
	    stroke(95);
	    fill(255);
	    strokeWeight(0.5);
	    rect(-2, 0, _wide+4, _high+_pad); 
	    point(-2,0);
	    point(_wide,0);
	    point(_wide,_high+_pad);
	    point(-2,_high+_pad);
	    
	    // Shadow
	    stroke(0);
	    strokeWeight(1);
	    line(-2, _high+_pad+2, _wide+3, _high+_pad+2);
	    line(_wide+3, _high+_pad+2, _wide+3, 0);
	    point(_wide+3,_high+_pad+2);
	    
	    // Draw the density function
	    if(_cdfOn) drawCDF(data,  numBins);
	    else       drawPDF(data, maxDen, imean, minDat, maxDat, numBins);
	    
	    // Draw the mean line
	    translate(0, -(_high+_pad));
	    stroke(178, 13, 13);	
	    double mean = ( (imean-minDat) / (maxDat-minDat) );
	    if(minDat == maxDat) mean = 0.5;
	    line( (float)mean*_wide, 0, (float)mean*_wide, _high + _pad);

	    // Label the mean, min & max
	    fill(0);
	    strokeWeight(2);
	    translate(0, _high+(_pad*0.5));
	    text((float)imean, (float)mean*_wide-15, 20);
	    stroke(0);	  
	    text((float)minDat, -15, 20);
	    text((float)maxDat, _wide-25, 20);
	    
	    popMatrix();
	}
	
    }

    // * * * Draw the PDF * * * //
    void drawPDF(double [] data, double maxDen, double imean, double minDat, double maxDat, int numBins){
	translate(0, _high+_pad-1);
	stroke(25);
	fill(175);
	strokeWeight(1);   
	beginShape();
	vertex(0,0);
	vertex(0,  -(float)(data[0]/maxDen)*_high);
	for(int i = 0; i < numBins; i++)
	    vertex(float(i)*(_wide/numBins), -(float)(data[i]/(maxDen))*_high);
	vertex(_wide, -(float)(data[numBins-1]/maxDen)*_high);
	vertex(_wide, 0);
	vertex(0, 0);
	endShape();
    }

    // * * * Draw the CDF * * * //
    void drawCDF(double [] data, int numBins){
	translate(0, _high+_pad-1);
	stroke(25);
	fill(175);
	strokeWeight(1);   
	beginShape();
	float total = 0;
	vertex(0,0);
	vertex(0,  -total);
	for(int i = 0; i < numBins; i++){
	    total += (float)data[i];
	    vertex(float(i)*(_wide/numBins), -total*_high);
	}
	vertex(_wide, -total*_high);
	vertex(numBins*(_wide/numBins), 0);
	endShape();
    }

}