//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Class to manage the side-by-side scene
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
class SideByScene{

    DataMesh meshLeft, meshRight;
    LeftColorbar colorbarLeft;
    RightColorbar colorbarRight;
    Pointer pointerLeft;
    Pointer pointerRight;

    // * * * Constructor * * * //
    SideByScene(){
	meshLeft = new DataMesh(); 
	meshRight = new DataMesh(new GrayscaleColormapFactory().getColormap());
	meshLeft.setOrigin(new Point(-235, 0, 0)); 
	meshRight.setOrigin(new Point(225, 0, 0));
  
	colorbarLeft = new LeftColorbar(meshLeft.getColormap());
	colorbarRight = new RightColorbar(meshRight.getColormap());
	colorbarLeft.setOrigin(meshLeft.getOrigin());

	pointerLeft  = new Pointer(new Point(), meshLeft.getOrigin());
	pointerRight = new Pointer(new Point(), meshRight.getOrigin());
    }

    // * * * Draw the scene * * * //
    void drawScene(Picker picker, DataObject object, boolean drawMesh, Point trans, Point rot){
	meshLeft.drawMesh(object, drawMesh, trans, rot);
	meshRight.drawMesh(object, drawMesh, trans, rot);  
	colorbarLeft.drawColorbar();
	colorbarRight.drawColorbar();
	picker.start(0);
  	pointerLeft.drawPointer(trans, rot);
	picker.start(1);
	pointerRight.drawPointer(trans, rot);
    }
    
    // * * * Draw the pointers * * * //
    void drawPointer(Picker picker, Point trans, Point rot)
    {
	pointerLeft.drawPointer(trans, rot);
	picker.start(1);
	pointerRight.drawPointer(trans, rot);
    }

    // * * * Set the pointer selected state * * * //
    void setSelected(boolean select){
    	pointerLeft.setSelected(select); 
	pointerRight.setSelected(select);
    }

    // * * * Set the colormap data * * * //
    void setColorData(double[] data1, double [] data2, 
		      double min1, double max1, 
		      double min2, double max2, 
		      String label1, String label2){
   
	meshLeft.setColorData(data1, min1, max1);
	colorbarLeft.setColormapData((float)min1, (float)max1);
	colorbarLeft.setLabel(label1);

	meshRight.setColorData(data2, min2, max2);
	colorbarRight.setColormapData((float)min2, (float)max2);
	colorbarRight.setLabel(label2);
    }
    
    // * * * Set the current colorbar point * * * //
    void setCurrentPt(Point locate, Point point) {
	colorbarRight.setCurrentPt(point.y());
	colorbarLeft.setCurrentPt(point.x()); 
	pointerRight.setLocate(locate);
	pointerLeft.setLocate(locate);
    }
}